use super::song::Song;
use rand::seq::SliceRandom;
use std::{cmp::min, collections::VecDeque, sync::Arc};
use tokio::sync::Mutex;
pub struct SongQueue {
    queue: Arc<Mutex<VecDeque<Song>>>,
}

impl SongQueue {
    pub fn new() -> SongQueue {
        SongQueue {
            queue: Arc::new(Mutex::new(VecDeque::new())),
        }
    }
    pub async fn push(&self, songs: Vec<Song>) {
        let mut queue = self.queue.lock().await;
        for item in songs.into_iter() {
            queue.push_back(item);
        }
    }
    pub async fn pop(&self) -> Option<Song> {
        let mut queue = self.queue.lock().await;
        queue.pop_front()
    }
    pub async fn shuffle(&self) -> Result<(), String> {
        let mut queue = self.queue.lock().await;
        if queue.len() == 0 {
            return Err("queue is empty".to_string());
        }
        queue.make_contiguous().shuffle(&mut rand::thread_rng());

        Ok(())
    }
    pub async fn clear(&self) -> Result<(), String> {
        let mut queue = self.queue.lock().await;
        if queue.len() == 0 {
            return Err("queue is empty".to_string());
        };
        queue.clear();
        Ok(())
    }
    pub async fn get_string(&self) -> String {
        let queue = self.queue.lock().await;
        if queue.len() == 0 {
            return "*empty*".to_string();
        };
        let mut s = String::new();
        s.push_str(&format!(
            "*Showing {} of {} songs*\n",
            min(20, queue.len()),
            queue.len()
        ));
        for (i, song) in queue.iter().take(20).enumerate() {
            s += &format!("{}: ", i);
            s += &song.get_string().await;
            s += "\n";
        }
        s
    }
}
